/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include "iogateway/AbstractMessageIOGateway.h"

AbstractMessageIOGateway :: AbstractMessageIOGateway(PortableDataIO * io)
      : _io(io), _hosed(false), _inputEnabled(true), _flushOnEmpty(true),
        _ioMessagePool(GetMessagePool()), _ioRefPool(GetMessageRefPool()), 
        _nextPulseAt(MUSCLE_NEVER_PULSE)
{
   // empty
}

AbstractMessageIOGateway :: ~AbstractMessageIOGateway() 
{
   delete _io;
}

void
AbstractMessageIOGateway ::
SetPools(PortableMessageRef::ItemPool * ioMessagePool, PortableMessageRef::RefPool * refPool)
{
   _ioMessagePool = ioMessagePool;
   _ioRefPool = refPool;
}

// Adds the given message reference to our list of outgoing messages
// to send.  Never blocks.  Returns false iff for some reason
// the message can't be queued (out of memory?)
status_t 
AbstractMessageIOGateway ::
AddOutgoingMessage(const PortableMessageRef & messageRef)
{
   if (_hosed) return B_ERROR;
   return _outgoingMessages.AddTail(messageRef) ? B_NO_ERROR : B_ERROR;
}


// Retrieves the next message from the incoming messages queue.
// Returns false iff the queue is empty (in which case setRef
// will not be modified) 
status_t 
AbstractMessageIOGateway ::
GetNextIncomingMessage(PortableMessageRef & setRef)
{
   return _incomingMessages.RemoveHead(setRef) ? B_NO_ERROR : B_ERROR;
}

bool 
AbstractMessageIOGateway ::
HasIncomingMessagesReady() const
{
   return (_incomingMessages.CountItems() > 0);
}

// Handles buffer allocation and re-allocation
bool
AbstractMessageIOGateway ::
EnsureBufferSize(uint8 ** bufPtr, size_t * bufSize, size_t desiredSize, size_t copySize)
{
   uint8 * oldBuffer = *bufPtr;
   size_t oldBufSize = *bufSize;

   if ((oldBuffer == NULL)||(oldBufSize < desiredSize))
   {
      uint8 * newBuffer = newnothrow uint8[desiredSize];
      if (newBuffer == NULL) {WARN_OUT_OF_MEMORY; return false;}

      *bufPtr  = newBuffer;
      *bufSize = desiredSize;

      if (copySize > 0) memcpy(newBuffer, oldBuffer, copySize);
      delete [] oldBuffer;
   }
   return true;
}

void
AbstractMessageIOGateway ::
SetFlushOnEmpty(bool f)
{
   _flushOnEmpty = f;
}

void
AbstractMessageIOGateway ::
SetInputEnabled(bool enable)
{
   _inputEnabled = enable;
}

bool
AbstractMessageIOGateway ::
GetInputEnabled() const
{
   return _inputEnabled;
}

uint64
AbstractMessageIOGateway ::
GetPulseTime(uint64 /*now*/)
{
   return MUSCLE_NEVER_PULSE;
}

void 
AbstractMessageIOGateway ::
ReschedulePulseTime(uint64 pulseTime)
{
   _nextPulseAt = pulseTime;
}

void 
AbstractMessageIOGateway ::
ReschedulePulseTime()
{
   _nextPulseAt = 0;
}

void 
AbstractMessageIOGateway ::
Pulse(uint64 /*now*/)
{
   // empty
}

uint64
AbstractMessageIOGateway ::
GetOutputStallLimit() const
{
   return _io ? _io->GetOutputStallLimit() : ((uint64)-1);
}
