/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include "iogateway/PortableMessageIOGateway.h"
#include "dataio/FileDataIO.h"

#define TEST(x) if ((x) != B_NO_ERROR) printf("Test failed, line %i\n",__LINE__)

// This program tests the functionality of the PortableMessageIOGateway by writing a PortableMessage
// out to a file, then reading it back in.
int main(int argc, char ** argv) 
{
   if (argc == 1)
   {
      FILE * f = fopen("test.dat", "wb");
      if (f)
      {
         printf("Outputting test messages to test.dat...\n");
         PortableMessageIOGateway g(new FileDataIO(f));
         for (int i=0; i<10; i++)
         {
            PortableMessage * m = new PortableMessage(MAKETYPE("TeSt"));
            TEST(m->AddString("Jo", "Mama"));
            TEST(m->AddInt32("Age", 90+i));
            TEST(m->AddBool("Ugly", (i%2)!=0));
            TEST(g.AddOutgoingMessage(PortableMessageRef(m, NULL, NULL)));
         }
         uint32 wroteBytes = 0;
         while(g.HasBytesToOutput()) TEST(g.DoOutput(wroteBytes));
         printf("Done Writing!\n");
      }
      else printf("Error, couldn't open test file!\n");

      // Now try to read it back in
      FILE * in = fopen("test.dat", "rb");
      if (in)
      {
         printf("Reading test messages from test.dat...\n");
         PortableMessageIOGateway g(new FileDataIO(in));
         uint32 readBytes = 0;
         while(g.DoInput(readBytes) == B_NO_ERROR) 
         {
            PortableMessageRef msgRef;
            while(g.GetNextIncomingMessage(msgRef) == B_NO_ERROR) msgRef()->PrintToStream();
         }

         printf("Done Reading!\n");
      }
      else printf("Error, couldn't re-open test file!\n");
   }
   else if (argc > 1)
   {
      for (int i=1; i<argc; i++)
      {
         FILE * in = fopen(argv[i], "rb");
         if (in)
         {
            printf("Reading message file %s...\n", argv[i]);
            PortableMessageIOGateway g(new FileDataIO(in));
            uint32 readBytes = 0;
            while(g.DoInput(readBytes) == B_NO_ERROR) 
            {
               PortableMessageRef msgRef;
               while(g.GetNextIncomingMessage(msgRef) == B_NO_ERROR) msgRef()->PrintToStream();
            }

            printf("Done Reading!\n");
         }
         else printf("Error, couldn't open file [%s]!\n", argv[i]);
      }
   }
   return 0;
}
